clc;
% addpath('../TOOLS');
wb1 = waitbar(0,'Please wait...');
tic;



L = 3;                 % Level number
nAngles = 5;            % Angle number for SHE
N_P = 3;                % Number of phases
m_step = 0.01;          % Modulation index step


%% ------------------------- PSO PARAMETERS ------------------------- %%

tol = 1e-7;            % Minimization tolerance
MaxIt = 5000;           % Maximum number of iterations
Inv_Prop = 1/MaxIt;     % Inverse factor for computation speed
nPop = 120;             % Particles number (each particle is a set of angles)
wmax = 0.8;             % Max value of inertia coeff.
wmin = 0.2;             % Min value of inertia coeff.
w0   = 0.4*pi/2;
c1 = 2;                 % Personal acceleration coeff
c2 = 1.2;               % Social acceleration coeff


%% -------------------------- PSO ALGORITHM ------------------------- %%

% Create state vector:
u  = get_u(nAngles, L);

% Create vector of harmonics to be eliminated:
h  = get_h(nAngles, N_P);

% Create vector of initial angles:
a0 = get_a0(nAngles, L);

% Creat results variables:
m_list  = m_step:m_step:1;
m_lgt   = length(m_list);
ANGLES  = zeros(nAngles, m_lgt);
TOL     = zeros(1, m_lgt);
IT_NBR = zeros(1, m_lgt);
HARMONICS  = zeros(nAngles,m_lgt);

% Problem definition
if(L==2)
    FUNC = @(x,m) cost_function_2L(x,m,u,h);
else
    FUNC = @(x,m) cost_function_ML(x,m,L,u,h);
end
AngleMin = 0;       % Angles lower boundaries
AngleMax = pi/2;    % Angles upper boundaries


%%
% Initialiazation
empty_particle.Position      = [];
empty_particle.Velocity      = [];
empty_particle.Cost          = [];
empty_particle.Best.Position = [];
empty_particle.Best.Cost     = [];
particles = repmat(empty_particle, nPop, 1);


% Loop for each modulation index
for m_i = 1:length(m_list)

    m = m_list(m_i);

    GlobalBest.Cost = inf;

    % Loop for each angles
    for i=1:nPop
        for j=1:nAngles
            if(m_i==1)
                particles(i).Position(j) = a0(j) - 0.2 + 0.4*rand(1);
            else
                particles(i).Position(j) = ANGLES(j,m_i-1) - 0.2 + 0.4*rand(1);
            end
        end
        particles(i).Position = max(particles(i).Position, AngleMin);
        particles(i).Position = min(particles(i).Position, AngleMax);
        particles(i).Velocity = zeros(1, nAngles);
        [particles(i).Cost,~] = FUNC(particles(i).Position,m);
        particles(i).Best.Position = particles(i).Position;
        particles(i).Best.Cost = particles(i).Cost;
        if particles(i).Best.Cost < GlobalBest.Cost
            GlobalBest = particles(i).Best;
        end  
    end



    % START OF LOOP
    it = 1;
    while(GlobalBest.Cost>tol && it<=MaxIt)
        if(it==1 || mod(it,10)==0)
           waitbar(m_i/length(m_list),wb1,sprintf('SHE, for m = %0.3f, it: %0.f,  tolerance: %0.f e-10',m,it, GlobalBest.Cost*10^10));
        end
        w = w0; %wmax - (wmax-wmin)/MaxIt * it;

        for i=1:nPop

            % Update Velocity
            particles(i).Velocity = w*particles(i).Velocity + c1*rand(1, nAngles).*(particles(i).Best.Position - particles(i).Position) + c2*rand(1, nAngles).*(GlobalBest.Position - particles(i).Position);

            % Update Position
            particles(i).Position = particles(i).Position + particles(i).Velocity;

            % Others limits
            for j = 2:nAngles
                if(particles(i).Position(j)<particles(i).Position(j-1))
                    particles(i).Position(j) = particles(i).Position(j-1);
                end
            end

            % Apply Lower and Upper Bound Limits
            particles(i).Position = max(particles(i).Position, AngleMin);
            particles(i).Position = min(particles(i).Position, AngleMax);

            % Cost evaluation
            [particles(i).Cost,~] = FUNC(particles(i).Position,m);

            % Update Personal Best
            if particles(i).Cost < particles(i).Best.Cost

               particles(i).Best.Position = particles(i).Position;
               particles(i).Best.Cost = particles(i).Cost;

               % Update Global Best
               if particles(i).Best.Cost < GlobalBest.Cost
                   GlobalBest = particles(i).Best;
               end
            end
        end
        it = it+1;
    end


    % Stock values
    for angle = 1:nAngles
        ANGLES(angle,m_i) = GlobalBest.Position(angle);
    end
    TOL(m_i) = GlobalBest.Cost;
    IT_NBR(m_i) = it;
    [~, HARMONICS(:,m_i)] = FUNC(GlobalBest.Position,m);
end
toc;
close(wb1)


%% ---------------------- HARMONICS PERFORMANCES ----------------------- %%

wb1 = waitbar(0,'Please wait...');

% Initialisation
PHASE_THD    = zeros(1,m_lgt);
PHASE_WTHD   = zeros(1,m_lgt);
LINE_THD    = zeros(1,m_lgt);
LINE_WTHD   = zeros(1,m_lgt);
Vdctot = 800;

% Computes time basis
[timeDSP, timeFPGA] = getTimes(50, 10e3, 1);

% Phase
phase = timeFPGA*100*pi;
v = get_v(u);

for m_i = 1:m_lgt
    waitbar(m_i/length(m_list),wb1,sprintf('Computation of THD and WTHD, for m = %0.3f',m_list(m_i)));
    
    % Get angles
    angles = ANGLES(:,m_i);
    
    % PWM generation
    PWMs = P3M(L, angles, v, phase);
    
    % Simulation of ideal inverter
    [Va, Vab] = inverter(PWMs, Vdctot);

    % Computation of harmonic performance
    [~, PHASE_THD(m_i), PHASE_WTHD(m_i)] = Performances(Va);
    [~, LINE_THD(m_i),  LINE_WTHD(m_i)] = Performances(Vab);
end
close(wb1)
%rmpath('../TOOLS');


%% ------------------------------- PLOTS ------------------------------- %%
close all;

figure(3)
subplot(2,3,1)
for i=1:nAngles
    plot(m_list, ANGLES(i,:)*180/pi,'-x')
    hold on
end
ylabel('angles [deg]')
xlabel('m [-]')
title('Values of angles')

subplot(2,3,2)
semilogy(m_list,TOL,'-x')
ylabel('tolerance [-]')
xlabel('m [-]')
title('Tolerance')
grid on

subplot(2,3,3)
plot(m_list,abs(HARMONICS(1,:)),'-x')
ylabel('fondamental [-]')
xlabel('m [-]')
title('Values of fondamental')
grid on

subplot(2,3,4)
for angle=2:nAngles
    semilogy(m_list,abs(HARMONICS(angle,:)),'-x')
    hold on
end
ylabel('harmonic [-]')
xlabel('modulation index m [-]')
title('Values of harmonics')
grid on

subplot(2,3,5)
plot(m_list,PHASE_THD,'-x')
ylabel('THD [%]')
xlabel('tolerance [-]')
grid on


subplot(2,3,6)
plot(m_list,PHASE_WTHD,'-x')
ylabel('WTHD [%]')
xlabel('tolerance [-]')
grid on


%% ----------------------------- STORE DATA ---------------------------- %%

% load('DATA_SHE.mat')

SET.m_list      = m_list;
SET.ANGLES      = ANGLES;
SET.PHASE_THD   = PHASE_THD;
SET.PHASE_WTHD  = PHASE_WTHD;
SET.LINE_THD    = LINE_THD;
SET.LINE_WTHD   = LINE_WTHD;
SET.TOL         = TOL;
SET.NBRE_IT     = IT_NBR;

DATA_SHE.L_list = [3 5 7 9 11];
DATA_SHE.(['SET_',num2str(L),'L']).NA_list = [3 5 7 9 11];
DATA_SHE.(['SET_',num2str(L),'L']).(['SET_',num2str(nAngles),'A']) = SET;
%%
save('DATA_SHE.mat', 'DATA_SHE')